﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using PCBI.Automation;
using System.Drawing.Drawing2D;
using System.IO;

namespace Demo9CSharp
{
    public partial class Form1 : Form
    {
        private IPCBIWindow mainWindowPCBI; //fill e.g. in load or on other button click with mainWindowPCBI = IAutomation.CreateNewPCBIWindow(true);
        private string pathForOutput = @"C:\Tests\Images\";
        public Form1()
        {
            InitializeComponent();
            IAutomation.IAutomationInit();

            mainWindowPCBI = IAutomation.CreateNewPCBIWindow(true);
        }

        private void buttonExportImage_Click(object sender, EventArgs e)
        {
            IStep step = mainWindowPCBI.GetCurrentStep();
            if (step == null)
            {
                MessageBox.Show("Please load a job first to use this demo.", "No Job Loaded!", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            ICMPLayer layerCMPsTop = step.GetCMPLayer(true);
            //exisits the compnenent layer?
            if (layerCMPsTop == null) return;
            
            List<IObject> layerObjectList = layerCMPsTop.GetAllLayerObjects();
            //is there a compnent on the layer?
            if (layerObjectList.Count == 0) return;

            if (!Directory.Exists(pathForOutput))
                Directory.CreateDirectory(pathForOutput);

            for(int i =0; i< layerObjectList.Count; i++) 
            {
                try
                {
                    ICMPObject cmp = (ICMPObject)layerObjectList[i];
 
                    SaveImageFromComponent(cmp, step, pathForOutput);
                    CreateImagpWithCMPLocation(cmp, step, pathForOutput);
                 }
                catch (Exception exImg)
                {
                    Console.WriteLine("There was an error " + exImg.ToString());
                }
            }
            //something went wrong?
            string errorLog = IAutomation.GetErrorLog();
            if (errorLog.Length > 0)
                System.Diagnostics.Debug.WriteLine(errorLog);
        }
        /// <summary>
        /// This example shows how to make an image of an component with connections on the signal layer below.
        /// </summary>
        /// <param name="CMPforImage">The relevant component</param>
        /// <param name="step">current step from the component</param>
        /// <param name="FileLocation">Where should we save the image?</param>
        public void SaveImageFromComponent(ICMPObject CMPforImage, IStep step, string FileLocation)
        {
            List<ILayer> layers = new List<ILayer>(); //for the image we need all layers who are visible

            ICMPLayer CMPlayer = step.GetCMPLayer(CMPforImage.PlacedTop);
            IODBLayer outsideSignalLayer = step.GetOutsideODBLayer(CMPforImage.PlacedTop);

            if (CMPlayer == null) return; //check the layer is existing? maybe wrong job?
            layers.Add(CMPlayer);

            if (outsideSignalLayer != null) //we want to see where are pins connected
                layers.Add(outsideSignalLayer);

            RectangleF boundsRelevantCMP = CMPforImage.GetBounds();
            boundsRelevantCMP.Inflate(100, 100); //show a little bit of the area around the component

            //create the image and save it as png
            PCBI.Automation.IStep.BitmapResultClass imageClass = step.GetBitmap(layers, boundsRelevantCMP, 500, 500);
            if (imageClass != null)
                imageClass.Image.Save(FileLocation + CMPforImage.Ref + ".png", System.Drawing.Imaging.ImageFormat.Png);

        }
        /// <summary>
        /// To show the position of the component on the board
        /// </summary>
        /// <param name="CMPforImage">The relevant component</param>
        /// <param name="step">current step from the component</param>
        /// <param name="FileLocation">Where should we save the image?</param>
        public void CreateImagpWithCMPLocation(ICMPObject CMPforImage, IStep step, string FileLocation)
        {
            Bitmap bmp = new Bitmap(500, 500);

            Graphics g = Graphics.FromImage(bmp);
            Matrix matrix = new Matrix();

            RectangleF boundsStep = step.GetBounds();
            boundsStep.Inflate(20, 20); //add border in black
            matrix.Translate(-boundsStep.X, -boundsStep.Y);
            matrix.Scale(bmp.Width / boundsStep.Width*0.9f, bmp.Height / boundsStep.Height*0.9f);

            g.Clear(Color.Black); //black background
            g.Transform = matrix;
            g.DrawPath(Pens.White, step.GetPCBOutline()); //draw board outline
            g.FillRectangle(Brushes.Red, CMPforImage.GetBounds()); //draw cmp position

            bmp.Save(FileLocation + CMPforImage.Ref + "_Map.png", System.Drawing.Imaging.ImageFormat.Png);
        }
    }
}
